// vim: sw=2 ts=2 expandtab

#include <SDL/SDL_ttf.h>

#include "graphics.h"

TTF_Font *font;

SDL_Surface* init_graphics(int w, int h) {
  SDL_Surface *screen;

  if(SDL_Init(SDL_INIT_VIDEO) != 0) {
    printf("Could not initialize SDL: %s\n", SDL_GetError());
    return NULL;
  }

  if((screen = SDL_SetVideoMode(w, h, 24, SDL_HWSURFACE | SDL_DOUBLEBUF)) == NULL) {
    printf("Could not set SDL video mode: %s\n", SDL_GetError());
    return NULL;
  }

  if(TTF_Init() == -1) {
    printf("TTF_Init failed!\n");
    return NULL;
  }

  if((font = TTF_OpenFont("/usr/share/fonts/dejavu/DejaVuSansMono.ttf", 20)) == NULL) {
    printf("TTF_OpenFont failed!\n");
    return NULL;
  }

  SDL_WM_SetCaption("RealTime FFT", "RealTime FFT");

  return screen;
}

// from the SDL docs
void set_pixel(SDL_Surface *surface, int x, int y, Uint32 pixel) {
  int bpp = surface->format->BytesPerPixel;
  /* Here p is the address to the pixel we want to set */
  Uint8 *p = (Uint8 *)surface->pixels + y * surface->pitch + x * bpp;

  switch(bpp) {
    case 1:
      *p = pixel;
      break;

    case 2:
      *(Uint16 *)p = pixel;
      break;

    case 3:
      if(SDL_BYTEORDER == SDL_BIG_ENDIAN) {
        p[0] = (pixel >> 16) & 0xff;
        p[1] = (pixel >> 8) & 0xff;
        p[2] = pixel & 0xff;
      } else {
        p[0] = pixel & 0xff;
        p[1] = (pixel >> 8) & 0xff;
        p[2] = (pixel >> 16) & 0xff;
      }
      break;

    case 4:
      *(Uint32 *)p = pixel;
      break;
  }
}

void draw_line(SDL_Surface *surf, uint32_t cl, int x0, int y0, int x1, int y1) {
  int dx = abs(x1-x0);
  int dy = abs(y1-y0);
  double err, e2;
  int sx, sy;

  if(x0 < x1) {
    sx = 1;
  } else {
    sx = -1;
  }

  if(y0 < y1) {
    sy = 1;
  } else {
    sy = -1;
    err = dx-dy;

    SDL_LockSurface(surf);

    while(1) {
      set_pixel(surf, x0, y0, cl);

      if(x0 == x1 && y0 == y1) {
        break;
      }
      e2 = 2*err;
      if(e2 > -dy) {
        err = err - dy;
        x0 = x0 + sx;
      }
      if(e2 <  dx) {
        err = err + dx;
        y0 = y0 + sy ;
      }
    }

    SDL_UnlockSurface(surf);
  }
}

void draw_text(SDL_Surface *surf, char *str, SDL_Color textCol, SDL_Color bgCol, int x, int y) {
  SDL_Rect textRect = {x, y, 0, 0};
  SDL_Surface *textImg = TTF_RenderText_Shaded(font, str, textCol, bgCol);
  SDL_BlitSurface(textImg, NULL, surf, &textRect);
  SDL_FreeSurface(textImg);
}
